@@ -33,7 +33,7 @@
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcelanguagemanager.h>
-#include <gtksourceview/gtksourcestylemanager.h>
+#include <gtksourceview/gtksourcestyleschememanager.h>
 #ifdef TEST_XML_MEM
 #include <libxml/xmlreader.h>
 #endif
@@ -80,6 +80,9 @@ static void       insert_spaces_toggled_cb       (GtkAction       *action,
 static void       tabs_toggled_cb                (GtkAction       *action,
 						  GtkAction       *current,
 						  gpointer         user_data);
+static void       indent_toggled_cb              (GtkAction       *action,
+						  GtkAction       *current,
+						  gpointer         user_data);
 
 static GtkWidget *create_view_window             (GtkSourceBuffer *buffer,
 						  GtkSourceView   *from);
@@ -99,7 +102,8 @@ static GtkActionEntry view_action_entries[] = {
 	{ "ViewMenu", NULL, "_View" },
 	{ "NewView", GTK_STOCK_NEW, "_New View", NULL,
 	  "Create a new view of the file", G_CALLBACK (new_view_cb) },
-	{ "TabsWidth", NULL, "_Tabs Width" },
+	{ "TabWidth", NULL, "_Tab Width" },
+	{ "IndentWidth", NULL, "I_ndent Width" },
 	{ "SmartHomeEnd", NULL, "_Smart Home/End" },
 	{ "DebugThing3", GTK_STOCK_FIND_AND_REPLACE, "Search and _Replace", "<control>R",
 	  "Search and Replace", G_CALLBACK (debug_thing_3_cb) },
@@ -112,7 +116,7 @@ static GtkToggleActionEntry toggle_entries[] = {
 	{ "ShowMarkers", NULL, "Show _Markers", NULL,
 	  "Toggle visibility of markers in the left margin",
 	  G_CALLBACK (markers_toggled_cb), FALSE },
-	{ "ShowMargin", NULL, "Show M_argin", NULL,
+	{ "ShowMargin", NULL, "Show Right M_argin", NULL,
 	  "Toggle visibility of right margin indicator",
 	  G_CALLBACK (margin_toggled_cb), FALSE },
 	{ "HlLine", NULL, "_Highlight Current Line", NULL,
@@ -126,12 +130,21 @@ static GtkToggleActionEntry toggle_entries[] = {
 	  G_CALLBACK (insert_spaces_toggled_cb), FALSE }
 };
 
-static GtkRadioActionEntry radio_entries[] = {
-	{ "TabsWidth4", NULL, "4", NULL, "Set tabulation width to 4 spaces", 4 },
-	{ "TabsWidth6", NULL, "6", NULL, "Set tabulation width to 6 spaces", 6 },
-	{ "TabsWidth8", NULL, "8", NULL, "Set tabulation width to 8 spaces", 8 },
-	{ "TabsWidth10", NULL, "10", NULL, "Set tabulation width to 10 spaces", 10 },
-	{ "TabsWidth12", NULL, "12", NULL, "Set tabulation width to 12 spaces", 12 }
+static GtkRadioActionEntry tabs_radio_entries[] = {
+	{ "TabWidth4", NULL, "4", NULL, "Set tabulation width to 4 spaces", 4 },
+	{ "TabWidth6", NULL, "6", NULL, "Set tabulation width to 6 spaces", 6 },
+	{ "TabWidth8", NULL, "8", NULL, "Set tabulation width to 8 spaces", 8 },
+	{ "TabWidth10", NULL, "10", NULL, "Set tabulation width to 10 spaces", 10 },
+	{ "TabWidth12", NULL, "12", NULL, "Set tabulation width to 12 spaces", 12 }
+};
+
+static GtkRadioActionEntry indent_radio_entries[] = {
+	{ "IndentWidthUnset", NULL, "Use Tab Width", NULL, "Set indent width same as tab width", -1 },
+	{ "IndentWidth4", NULL, "4", NULL, "Set indent width to 4 spaces", 4 },
+	{ "IndentWidth6", NULL, "6", NULL, "Set indent width to 6 spaces", 6 },
+	{ "IndentWidth8", NULL, "8", NULL, "Set indent width to 8 spaces", 8 },
+	{ "IndentWidth10", NULL, "10", NULL, "Set indent width to 10 spaces", 10 },
+	{ "IndentWidth12", NULL, "12", NULL, "Set indent width to 12 spaces", 12 }
 };
 
 static GtkRadioActionEntry smart_home_end_entries[] = {
@@ -164,12 +177,20 @@ static const gchar *view_ui_description =
 "      <menuitem action=\"AutoIndent\"/>"
 "      <menuitem action=\"InsertSpaces\"/>"
 "      <separator/>"
-"      <menu action=\"TabsWidth\">"
-"        <menuitem action=\"TabsWidth4\"/>"
-"        <menuitem action=\"TabsWidth6\"/>"
-"        <menuitem action=\"TabsWidth8\"/>"
-"        <menuitem action=\"TabsWidth10\"/>"
-"        <menuitem action=\"TabsWidth12\"/>"
+"      <menu action=\"TabWidth\">"
+"        <menuitem action=\"TabWidth4\"/>"
+"        <menuitem action=\"TabWidth6\"/>"
+"        <menuitem action=\"TabWidth8\"/>"
+"        <menuitem action=\"TabWidth10\"/>"
+"        <menuitem action=\"TabWidth12\"/>"
+"      </menu>"
+"      <menu action=\"IndentWidth\">"
+"        <menuitem action=\"IndentWidthUnset\"/>"
+"        <menuitem action=\"IndentWidth4\"/>"
+"        <menuitem action=\"IndentWidth6\"/>"
+"        <menuitem action=\"IndentWidth8\"/>"
+"        <menuitem action=\"IndentWidth10\"/>"
+"        <menuitem action=\"IndentWidth12\"/>"
 "      </menu>"
 "      <separator/>"
 "      <menu action=\"SmartHomeEnd\">"
@@ -284,7 +305,7 @@ remove_all_markers (GtkSourceBuffer *buffer)
 static GtkSourceLanguage *
 get_language_for_filename (const gchar *filename)
 {
-	GSList *list;
+	const gchar * const *languages;
 	gchar *filename_utf8;
 	GtkSourceLanguageManager *manager;
 
@@ -292,15 +313,17 @@ get_language_for_filename (const gchar *filename)
 	g_return_val_if_fail (filename_utf8 != NULL, NULL);
 
 	manager = gtk_source_language_manager_get_default ();
-	list = gtk_source_language_manager_list_languages (manager);
+	languages = gtk_source_language_manager_get_language_ids (manager);
 
-	while (list != NULL)
+	while (*languages != NULL)
 	{
 		GtkSourceLanguage *lang;
 		gchar **globs, **p;
 
-		lang = list->data;
-		list = g_slist_delete_link (list, list);
+		lang = gtk_source_language_manager_get_language (manager,
+								 *languages);
+		g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE (lang), NULL);
+		++languages;
 
 		globs = gtk_source_language_get_globs (lang);
 		if (globs == NULL)
@@ -312,7 +335,7 @@ get_language_for_filename (const gchar *filename)
 			{
 				g_strfreev (globs);
 				g_free (filename_utf8);
-				g_slist_free (list);
+
 				return lang;
 			}
 		}
@@ -330,19 +353,21 @@ get_language_for_filename (const gchar *filename)
 static GtkSourceLanguage *
 get_language_for_mime_type (const gchar *mime)
 {
-	GSList *list;
+	const gchar * const *languages;
 	GtkSourceLanguageManager *manager;
 
 	manager = gtk_source_language_manager_get_default ();
-	list = gtk_source_language_manager_list_languages (manager);
+	languages = gtk_source_language_manager_get_language_ids (manager);
 
-	while (list != NULL)
+	while (*languages != NULL)
 	{
 		GtkSourceLanguage *lang;
 		gchar **mimetypes, **p;
 
-		lang = list->data;
-		list = g_slist_delete_link (list, list);
+		lang = gtk_source_language_manager_get_language (manager,
+								 *languages);
+		g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE (lang), NULL);
+		++languages;
 
 		mimetypes = gtk_source_language_get_mime_types (lang);
 
@@ -354,7 +379,6 @@ get_language_for_mime_type (const gchar *mime)
 			if (strcmp (*p, mime) == 0)
 			{
 				g_strfreev (mimetypes);
-				g_slist_free (list);
 				return lang;
 			}
 		}
@@ -408,7 +432,7 @@ open_file (GtkSourceBuffer *buffer, const gchar *filename)
 	GtkSourceLanguage *language = NULL;
 	gchar *freeme = NULL;
 	gboolean success = FALSE;
-		
+
 	if (!g_path_is_absolute (filename))
 	{
 		gchar *curdir = g_get_current_dir ();
@@ -437,32 +461,32 @@ open_file (GtkSourceBuffer *buffer, const gchar *filename)
 	if (language != NULL)
 	{
 		gchar **styles;
-		
+
 		styles = gtk_source_language_get_style_ids (language);
-		
+
 		if (styles == NULL)
 			g_print ("No styles in language '%s'\n", gtk_source_language_get_name (language));
-		else 
+		else
 		{
 			gchar **ids;
 			g_print ("Styles in in language '%s':\n", gtk_source_language_get_name (language));
-			
+
 			ids = styles;
-			
-			while (*ids != NULL) 
+
+			while (*ids != NULL)
 			{
 				const gchar *name;
-				
+
 				name = gtk_source_language_get_style_name (language, *ids);
 
 				g_print ("- %s (name: '%s')\n", *ids, name);
 
 				++ids;
 			}
-			
-			g_strfreev (styles);		
-		}	
-		
+
+			g_strfreev (styles);
+		}
+
 		g_print("\n");
 	}
 out:
@@ -495,7 +519,7 @@ static void
 margin_toggled_cb (GtkAction *action, gpointer user_data)
 {
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_IS_SOURCE_VIEW (user_data));
-	gtk_source_view_set_show_margin (
+	gtk_source_view_set_show_right_margin (
 		GTK_SOURCE_VIEW (user_data),
 		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
@@ -509,7 +533,6 @@ hl_line_toggled_cb (GtkAction *action, gpointer user_data)
 		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
-
 static void
 auto_indent_toggled_cb (GtkAction *action,
 			gpointer   user_data)
@@ -536,7 +559,18 @@ tabs_toggled_cb (GtkAction *action,
 		 gpointer user_data)
 {
 	g_return_if_fail (GTK_IS_RADIO_ACTION (action) && GTK_IS_SOURCE_VIEW (user_data));
-	gtk_source_view_set_tabs_width (
+	gtk_source_view_set_tab_width (
+		GTK_SOURCE_VIEW (user_data),
+		gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action)));
+}
+
+static void
+indent_toggled_cb (GtkAction *action,
+		   GtkAction *current,
+		   gpointer user_data)
+{
+	g_return_if_fail (GTK_IS_RADIO_ACTION (action) && GTK_IS_SOURCE_VIEW (user_data));
+	gtk_source_view_set_indent_width (
 		GTK_SOURCE_VIEW (user_data),
 		gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action)));
 }
@@ -722,7 +756,7 @@ update_cursor_position (GtkTextBuffer *buffer, gpointer user_data)
 	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
 
 	view = GTK_SOURCE_VIEW (user_data);
-	tabwidth = gtk_source_view_get_tabs_width (view);
+	tabwidth = gtk_source_view_get_tab_width (view);
 	pos_label = GTK_LABEL (g_object_get_data (G_OBJECT (view), "pos_label"));
 
 	gtk_text_buffer_get_iter_at_mark (buffer,
@@ -921,9 +955,12 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 				      G_N_ELEMENTS (view_action_entries), view);
 	gtk_action_group_add_toggle_actions (action_group, toggle_entries,
 					     G_N_ELEMENTS (toggle_entries), view);
-	gtk_action_group_add_radio_actions (action_group, radio_entries,
-					    G_N_ELEMENTS (radio_entries),
+	gtk_action_group_add_radio_actions (action_group, tabs_radio_entries,
+					    G_N_ELEMENTS (tabs_radio_entries),
 					    -1, G_CALLBACK (tabs_toggled_cb), view);
+	gtk_action_group_add_radio_actions (action_group, indent_radio_entries,
+					    G_N_ELEMENTS (indent_radio_entries),
+					    -1, G_CALLBACK (indent_toggled_cb), view);
 	gtk_action_group_add_radio_actions (action_group, smart_home_end_entries,
 					    G_N_ELEMENTS (smart_home_end_entries),
 					    -1, G_CALLBACK (smart_home_end_toggled_cb), view);
@@ -975,6 +1012,7 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 	if (from)
 	{
 		gchar *tmp;
+		gint i;
 		GtkAction *action;
 
 		action = gtk_action_group_get_action (action_group, "ShowNumbers");
@@ -987,7 +1025,7 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 
 		action = gtk_action_group_get_action (action_group, "ShowMargin");
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-					      gtk_source_view_get_show_margin (from));
+					      gtk_source_view_get_show_right_margin (from));
 
 		action = gtk_action_group_get_action (action_group, "HlLine");
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
@@ -1002,8 +1040,14 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 			GTK_TOGGLE_ACTION (action),
 			gtk_source_view_get_insert_spaces_instead_of_tabs (from));
 
+		tmp = g_strdup_printf ("TabWidth%d", gtk_source_view_get_tab_width (from));
+		action = gtk_action_group_get_action (action_group, tmp);
+		if (action)
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+		g_free (tmp);
 
-		tmp = g_strdup_printf ("TabsWidth%d", gtk_source_view_get_tabs_width (from));
+		i = gtk_source_view_get_indent_width (from);
+		tmp = i < 0 ? g_strdup ("IndentWidthUnset") : g_strdup_printf ("IndentWidth%d", i);
 		action = gtk_action_group_get_action (action_group, tmp);
 		if (action)
 			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
@@ -1091,7 +1135,10 @@ create_main_window (GtkSourceBuffer *buffer)
 	action = gtk_action_group_get_action (action_group, "InsertSpaces");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
 
-	action = gtk_action_group_get_action (action_group, "TabsWidth8");
+	action = gtk_action_group_get_action (action_group, "TabWidth8");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+
+	action = gtk_action_group_get_action (action_group, "IndentWidthUnset");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
 	return window;
@@ -1183,7 +1230,7 @@ main (int argc, char *argv[])
 {
 	GtkWidget *window;
 	GtkSourceLanguageManager *lm;
-	GtkSourceStyleManager *sm;
+	GtkSourceStyleSchemeManager *sm;
 	GtkSourceBuffer *buffer;
 
 	gchar *builtin_lang_dirs[] = {TOP_SRCDIR "/gtksourceview/language-specs", NULL};
@@ -1225,17 +1272,24 @@ main (int argc, char *argv[])
 
 	lang_dirs = use_default_paths ? NULL : builtin_sm_dirs;
 
-	sm = gtk_source_style_manager_get_default ();
-	gtk_source_style_manager_set_search_path (sm, lang_dirs);
+	sm = gtk_source_style_scheme_manager_get_default ();
+	gtk_source_style_scheme_manager_set_search_path (sm, lang_dirs);
 
 	if (!use_default_paths)
-		gtk_source_style_manager_append_search_path (sm, TOP_SRCDIR "/tests/test-scheme.xml");
+		gtk_source_style_scheme_manager_append_search_path (sm, TOP_SRCDIR "/tests/test-scheme.xml");
 
-	schemes = gtk_source_style_manager_get_scheme_ids (sm);
+	schemes = gtk_source_style_scheme_manager_get_scheme_ids (sm);
 	g_print ("Available style schemes:\n");
 	while (*schemes != NULL)
 	{
-		style_scheme = gtk_source_style_manager_get_scheme (sm, *schemes);
+		const gchar* const *authors;
+		gchar *authors_str = NULL;
+
+		style_scheme = gtk_source_style_scheme_manager_get_scheme (sm, *schemes);
+
+		authors = gtk_source_style_scheme_get_authors (style_scheme);
+		if (authors != NULL)
+			authors_str = g_strjoinv (", ", (gchar **)authors);
 
 		g_print (" - [%s] %s: %s\n",
 			 gtk_source_style_scheme_get_id (style_scheme),
@@ -1243,12 +1297,17 @@ main (int argc, char *argv[])
 			 gtk_source_style_scheme_get_description (style_scheme) ?
 				gtk_source_style_scheme_get_description (style_scheme) : "");
 
+		if (authors_str) {
+			g_print ("   by %s\n",  authors_str);
+			g_free (authors_str);
+		}
+
 		++schemes;
 	}
 	g_print("\n");
 
 	if (style_scheme_id != NULL)
-		style_scheme = gtk_source_style_manager_get_scheme (sm, style_scheme_id);
+		style_scheme = gtk_source_style_scheme_manager_get_scheme (sm, style_scheme_id);
 	else
 		style_scheme = NULL;
 
